@@ -40,6 +40,7 @@ tiledb_config.from_ptr <- function(ptr) {
 #' @param config (optonal) character vector of config parameter names, values
 #' @return `tiledb_config` object
 #' @examples
+#' \dontshow{ctx <- tiledb_ctx(limitTileDBCores())}
 #' cfg <- tiledb_config()
 #' cfg["sm.tile_cache_size"]
 #'
@@ -70,6 +71,7 @@ tiledb_config <- function(config = NA_character_) {
 #' @param drop Optional logical switch to drop dimensions, default FALSE, currently unused.
 #' @return a config string value if parameter exists, else NA
 #' @examples
+#' \dontshow{ctx <- tiledb_ctx(limitTileDBCores())}
 #' cfg <- tiledb_config()
 #' cfg["sm.tile_cache_size"]
 #' cfg["does_not_exist"]
@@ -97,6 +99,7 @@ setMethod("[", "tiledb_config",
 #' @param value value to set, will be converted into a stringa
 #' @return updated `tiledb_config` object
 #' @examples
+#' \dontshow{ctx <- tiledb_ctx(limitTileDBCores())}
 #' cfg <- tiledb_config()
 #' cfg["sm.tile_cache_size"]
 #'
@@ -131,6 +134,7 @@ setMethod("[<-", "tiledb_config",
 #'
 #' @param object `tiledb_config` object
 #' @examples
+#' \dontshow{ctx <- tiledb_ctx(limitTileDBCores())}
 #' cfg <- tiledb_config()
 #' show(cfg)
 #' @export
@@ -145,6 +149,7 @@ setMethod("show", signature(object = "tiledb_config"),
 #' @param path The path to config file to be created
 #' @return path to created config file
 #' @examples
+#' \dontshow{ctx <- tiledb_ctx(limitTileDBCores())}
 #' tmp <- tempfile()
 #' cfg <- tiledb_config(c("sm.tile_cache_size" = "10"))
 #' pth <- tiledb_config_save(cfg, tmp)
@@ -162,6 +167,7 @@ tiledb_config_save <- function(config, path) {
 #'
 #' @param path path to the config file
 #' @examples
+#' \dontshow{ctx <- tiledb_ctx(limitTileDBCores())}
 #' tmp <- tempfile()
 #' cfg <- tiledb_config(c("sm.tile_cache_size" = "10"))
 #' pth <- tiledb_config_save(cfg, tmp)
@@ -181,6 +187,7 @@ tiledb_config_load <- function(path) {
 #' @param mode Character value `"any"`, currently unused
 #' @return a character vector of config parameter names, values
 #' @examples
+#' \dontshow{ctx <- tiledb_ctx(limitTileDBCores())}
 #' cfg <- tiledb_config()
 #' as.vector(cfg)
 #'
@@ -204,3 +211,37 @@ as.data.frame.tiledb_config <- function(x, ...) {
   values <- as.vector(v)
   return(data.frame("parameter" = params, "value" = values, stringsAsFactors = FALSE))
 }
+
+#' Limit TileDB core use to a given number of cores
+#'
+#' By default, TileDB will use all available cores on a given machine. In multi-user or
+#' multi-process settings, one may want to reduce the number of core. This function will
+#' take a given number, or default to smaller of the \sQuote{Ncpus} options value or the
+#' \sQuote{"OMP_THREAD_LIMIT"} enviroment variable (or two as hard fallback).
+#' @param ncores Value of CPUs used, if missing the smaller of a fallback of two, the value of
+#' \sQuote{Ncpus} (if set) and the value of environment variable \sQuote{"OMP_THREAD_LIMIT"} is
+#' used.
+#' @param verbose Optional logical toggle; if set, a short message is displayed informing the
+#' user about the value set.
+#' @return The modified configuration object is returned invisibly. As a side-effect the
+#' updated config is also used to set the global context object.
+#' @importFrom stats na.omit
+#' @export
+limitTileDBCores <- function(ncores, verbose=FALSE) {
+  if (missing(ncores)) {
+    ## start with a simple fallback: 'Ncpus' (if set) or else 2
+    ncores <- getOption("Ncpus", 2L)
+    ## also consider OMP_THREAD_LIMIT (cf Writing R Extensions), gets NA if envvar unset
+    ompcores <- as.integer(Sys.getenv("OMP_THREAD_LIMIT"))
+    ## and then keep the smaller
+    ncores <- min(na.omit(c(ncores, ompcores)))
+  }
+  cfg <- tiledb_config()
+  cfg["sm.num_reader_threads"] <- ncores
+  #cfg["sm.num_tbb_threads"] <- ncores
+  cfg["sm.num_writer_threads"] <- ncores
+  cfg["vfs.file.max_parallel_ops"] <- ncores
+  cfg["vfs.num_threads"] <- ncores
+  if (verbose) message("Limiting TileDB to ",ncores," cores. See ?limitTileDBCores.")
+  invisible(cfg)
+}
